﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using NerdDinner.Helpers;

using NerdDinner.Models;

namespace NerdDinner.Controllers
{
    public class DinnersController : Controller
    {

        DinnerRepository dinnerRepository = new DinnerRepository();

        //
        // GET: /Dinners/
        //      /Dinners/Page/2

        public ActionResult Index(int page = 0) {

            const int pageSize = 10;

            var upcomingDinners = dinnerRepository.FindUpcomingDinners();
            var paginatedDinners = new PaginatedList<Dinner>(upcomingDinners,
                                                            page,
                                                            pageSize);
            return View(paginatedDinners);
        }

        // GET: /Dinners/Details/2

        public ActionResult Details(int id) {
            Dinner dinner = dinnerRepository.GetDinner(id);
            
            if (dinner == null)
                return View("NotFound");
            else
                return View(dinner);
        }

        // GET: /Dinners/Edit/2

        [Authorize]
        public ActionResult Edit(int id) {
            Dinner dinner = dinnerRepository.GetDinner(id);

            if (!dinner.IsHostedBy(User.Identity.Name))
                return View("InvalidOwner");

            return View(new DinnerFormViewModel(dinner));
        }

        // POST: /Dinners/Edit/2

        [HttpPost, Authorize]
        public ActionResult Edit(int id, FormCollection formValues) { 
            //Retrieve existing dinner
            Dinner dinner = dinnerRepository.GetDinner(id);

            if (!dinner.IsHostedBy(User.Identity.Name))
                return View("InvalidOwner");

            //Update dinner with form posted values 
            try
            {
                UpdateModel(dinner, "Dinner");

                //Persist changes back to database
                dinnerRepository.Save();

                //Perform HTTP redirect to details page for the saved Dinner
                return RedirectToAction("Details", new { id = dinner.DinnerID });
            }
            catch
            {
                return View(new DinnerFormViewModel(dinner));
            }
        }

        // GET: /Dinners/Create

        [Authorize]
        public ActionResult Create() {
            Dinner dinner = new Dinner() {
                EventDate = DateTime.Now.AddDays(7)
            };

            return View(new DinnerFormViewModel(dinner));
        }

        // POST: /Dinners/Create

        [HttpPost, Authorize]
        public ActionResult Create(Dinner dinner) {
            if (ModelState.IsValid) {
                dinner.HostedBy = User.Identity.Name;

                RSVP rsvp = new RSVP();
                rsvp.AttendeeName = User.Identity.Name;
                dinner.RSVPs.Add(rsvp);

                dinnerRepository.Add(dinner);
                dinnerRepository.Save();

                return RedirectToAction("Details", new { id=dinner.DinnerID});
            }
            return View(new DinnerFormViewModel(dinner));
        }

        // HTTP GET: /Dinners/Delete/1
        [Authorize]
        public ActionResult Delete(int id) {
            Dinner dinner = dinnerRepository.GetDinner(id);

            if (!dinner.IsHostedBy(User.Identity.Name))
                return View("InvalidOwner");

            if (dinner == null)
                return View("NotFound");
            
            return View(dinner);
        }

        // HTTP POST: /Dinners/Delete/1

        [HttpPost, Authorize]
        public ActionResult Delete(int id, string confirmButton) {

            Dinner dinner = dinnerRepository.GetDinner(id);

            if (!dinner.IsHostedBy(User.Identity.Name))
                return View("InvalidOwner");

            if (dinner == null)
                return View("NotFound");

            dinnerRepository.Delete(dinner);
            dinnerRepository.Save();

            return View("Deleted");
        }

    }
}
